@@ -36,7 +36,7 @@ sub __stmt_fold {
         }
         my $in = ' ' x $sf->{o}{G}{base_indent};
         my %tabs = ( init_tab => $in x $indent, subseq_tab => $in x ( $indent + 1 ) );
-        return line_fold( $stmt, $term_w, { %tabs, join => 0 } ); ##
+        return line_fold( $stmt, $term_w, { %tabs, join => 0 } );
     }
     else {
         return $stmt;
@@ -63,18 +63,12 @@ sub get_stmt {
     my $indent2 = 2;
     my $qt_table = $sql->{table};
     my @tmp;
-    if ( defined $sf->{d}{ctes} && @{$sf->{d}{ctes}} ) { ##
-        #if ( @{$sf->{d}{ctes}} == 1 ) {
-        #    my $cte = $sf->{d}{ctes}[0];
-        #    push @tmp, $sf->__stmt_fold( $used_for, sprintf( 'WITH %s AS (%s)', $cte->{name}, $cte->{query} ), $indent0 );
-        #}
-        #else {
-            push @tmp, "WITH";
-            for my $cte ( @{$sf->{d}{ctes}} ) {
-                push @tmp, $sf->__stmt_fold( $used_for, sprintf( '%s AS (%s),', $cte->{name}, $cte->{query} ), $indent1 );
-            }
-            $tmp[-1] =~ s/,\z//;
-        #}
+    if ( defined $sql->{ctes} && @{$sql->{ctes}} ) {
+        push @tmp, "WITH";
+        for my $cte ( @{$sql->{ctes}} ) {
+            push @tmp, $sf->__stmt_fold( $used_for, sprintf( '%s AS (%s),', $cte->{name}, $cte->{query} ), $indent1 );
+        }
+        $tmp[-1] =~ s/,\z//;
         push @tmp, " ";
     }
     if ( $sql->{case_stmt} ) {
@@ -239,7 +233,7 @@ sub __prepare_table_row {
     my $dots = $sf->{i}{dots};
     my $dots_w = print_columns( $dots );
     no warnings 'uninitialized';
-    my $row_str = join( $list_sep, map { s/\t/  /g; s/\n/[NL]/g; s/\v/[VWS]/g; $_ } @$row );
+    my $row_str = join( $list_sep, map { s/\t/  /g; s/\n/\\n/g; s/\v/\\v/g; $_ } @$row );
     return unicode_sprintf( $indent . $row_str, $term_w, { mark_if_truncated => [ $dots, $dots_w ] } );
 }
 
@@ -314,7 +308,13 @@ sub alias {
     my ( $sf, $sql, $type, $identifier, $default ) = @_;
     my $prompt = 'as ';
     my $alias;
-    if ( $sf->{o}{alias}{$type} ) {
+    if ( $sf->{o}{alias}{$type} == 0 ) {
+        return;
+    }
+    elsif ( $sf->{o}{alias}{$type} == 1 ) {
+        return $default;
+    }
+    elsif ( $sf->{o}{alias}{$type} == 2 || $sf->{o}{alias}{$type} == 3 ) {
         my $tr = Term::Form::ReadLine->new( $sf->{i}{tr_default} );
         my $info = $sf->get_sql_info( $sql );
         if ( length $identifier ) {
@@ -331,11 +331,11 @@ sub alias {
             { info => $info, history => [ 'a' .. 'z' ] }
         );
         $sf->print_sql_info( $info );
+        if ( $sf->{o}{alias}{$type} == 3 && ! length $alias ) {
+            $alias = $default;
+        }
+        return $alias;
     }
-    if ( ! length $alias ) {
-        $alias = $default;
-    }
-    return $alias;
 }
 
 
@@ -393,7 +393,7 @@ sub quote_column {
     my ( $sf, @id ) = @_;
     if ( @id == 2 ) { # join: table_alias.column_name
         return $sf->__qualify_identifier(
-            $sf->__quote_identifier( 'aliases', shift @id ), $sf->__quote_identifier( 'columns', shift @id )
+            $sf->__quote_identifier( 'aliases', shift @id ), $sf->__quote_identifier( 'columns', shift @id ) ##
         );
     }
     else {
@@ -422,34 +422,16 @@ sub unquote_identifier {
 }
 
 
-sub backup_href {
-    my ( $sf, $href ) = @_;
-    my $backup = {};
-    for ( keys %$href ) {
-        if ( ref $href->{$_} eq 'ARRAY' ) {
-            $backup->{$_} = [ @{$href->{$_}} ];
-        }
-        elsif ( ref $href->{$_} eq 'HASH' ) {
-            $backup->{$_} = { %{$href->{$_}} };
-        }
-        else {
-            $backup->{$_} = $href->{$_};
-        }
-    }
-    return $backup;
-}
-
-
 sub reset_sql {
     my ( $sf, $sql ) = @_;
     my $backup = {};
     for my $y ( qw( db schema table cols ) ) {
         $backup->{$y} = $sql->{$y} if exists $sql->{$y};
     }
-    map { delete $sql->{$_} } keys %$sql; # not "$sql = {}" so $sql is still pointing to the outer $sql
+    delete @{$sql}{ keys %$sql }; # not "$sql = {}" so $sql is still pointing to the outer $sql
     my @string = qw( distinct_stmt set_stmt where_stmt group_by_stmt having_stmt order_by_stmt limit_stmt offset_stmt );
     my @array  = qw( cols group_by_cols aggr_cols selected_cols insert_col_names create_table_col_names
-                     set_args where_args having_args insert_args );
+                     set_args insert_args ctes );
     my @hash   = qw( alias );
     @{$sql}{@string} = ( '' ) x  @string;
     @{$sql}{@array}  = map{ [] } @array;
@@ -487,18 +469,18 @@ sub sql_limit {
 
 
 sub column_names {
-    my ( $sf, $qt_table ) = @_;
+    my ( $sf, $qt_table, $ctes ) = @_;
     # without `LIMIT 0` slower with big tables: mysql, MariaDB and Pg
     # no difference with SQLite, Firebird, DB2 and Informix
     my $columns;
     if ( ! eval {
         my $stmt = '';
-        if ( defined $sf->{d}{ctes} && @{$sf->{d}{ctes}} ) {
-            $stmt = "WITH " . join ', ', map { sprintf '%s AS (%s)', $_->{name}, $_->{query} } @{$sf->{d}{ctes}};
+        if ( defined $ctes && @$ctes ) {
+            $stmt = "WITH " . join ', ', map { sprintf '%s AS (%s)', $_->{name}, $_->{query} } @$ctes;
             $stmt .= ' ';
         }
+
         $stmt .= "SELECT * FROM " . $qt_table . $sf->sql_limit( 0 );
-        #my $stmt = $sf->get_stmt( $sql, 'Select', 'prepare' ) . $sf->sql_limit( 0 ); #  $sql
         my $sth = $sf->{d}{dbh}->prepare( $stmt );
         if ( $sf->{i}{driver} ne 'SQLite' ) {
             $sth->execute();
@@ -542,50 +524,6 @@ sub read_json {
     ) {
         die "In '$file_fs':\n$@";
     }
-
-############################################################## 2.317  12.03.2023
-    if ( $file_fs eq ( $sf->{i}{f_attached_db} // '' ) ) {
-        my @keys = keys %$ref;
-        if ( ref( $ref->{$keys[0]} ) eq 'ARRAY' ) {
-            my $tmp;
-            for my $key ( @keys ) {
-                for my $ar ( @{$ref->{$key}} ) {
-                    $tmp->{$key}{$ar->[1]} = $ar->[0];
-                }
-            }
-            $sf->write_json( $sf->{i}{f_attached_db}, $tmp );
-            return $tmp;
-        }
-        #else {
-        #    return $ref;
-        #}
-    }
-##############################################################
-
-################################################################################################# 2.314  03.02.2023
-    if ( $file_fs eq ( $sf->{i}{f_subqueries} // '' ) ) {
-        my $tmp;
-        CONVERT: for my $driver ( keys %$ref ) {
-            for my $db ( keys %{$ref->{$driver}} ) {
-                last CONVERT if ref( $ref->{$driver}{$db} ) ne 'HASH';
-                for my $key ( keys %{$ref->{$driver}{$db}} ) {
-                    next if $key ne 'substmt';
-                    for my $ref ( @{$ref->{$driver}{$db}{$key}} ) {
-                        push @{$tmp->{$driver}{$db}}, { stmt => $ref->[0], name => $ref->[1] };
-                    }
-                }
-            }
-        }
-        if ( defined $tmp ) {
-            $sf->write_json( $sf->{i}{f_subqueries}, $tmp );
-            return $tmp;
-        }
-        #else {
-        #    return $ref;
-        #}
-    }
-##################################################################################################
-
     return $ref;
 }
 
