@@ -5,7 +5,7 @@ use warnings;
 use strict;
 use 5.014;
 
-our $VERSION = '2.344';
+our $VERSION = '2.400';
 
 #use bytes; # required
 use Scalar::Util qw( looks_like_number );
@@ -254,7 +254,7 @@ sub tables_info { # not documented
     my ( @user_table_keys, @sys_table_keys );
     my $tables_info = {};
     for my $info_table ( @$info_tables ) {
-        if ( $driver =~ /^(?:Informix|Sybase)\z/ && $schema ne $info_table->{$table_schem} ) {
+        if ( $driver =~ /^(?:Informix|Sybase)\z/ && defined $schema && $schema ne $info_table->{$table_schem} ) {
             # Informix: `table_info` returns everything.
             next;
         }
@@ -324,7 +324,7 @@ App::DBBrowser::DB - Database plugin documentation.
 
 =head1 VERSION
 
-Version 2.344
+Version 2.400
 
 =head1 DESCRIPTION
 
