@@ -30,7 +30,7 @@ sub new {
             " = ", " != ", " <> ", " < ", " > ", " >= ", " <= ",
             "REGEXP", "REGEXP_i", "NOT REGEXP", "NOT REGEXP_i", "LIKE", "NOT LIKE",
             "IS NULL", "IS NOT NULL", "IN", "NOT IN", "BETWEEN", "NOT BETWEEN",
-            "ANY", "ALL"
+            "ANY", "ALL",
         ],
     }, $class;
 }
@@ -117,11 +117,11 @@ sub _options {
             { name => '_file_encoding_out', text => "- File encoding out",  section => 'export'  },
         ],
         group_misc => [
-            { name => '_menu_memory',   text => "- Menu memory",  section => 'G'     },
-            { name => '_table_expand',  text => "- Expand table", section => 'table' },
-            { name => '_search',        text => "- Search",       section => 'table' },
-            { name => '_mouse',         text => "- Mouse mode",   section => 'table' },
-            { name => '_db2_encoding',  text => "- DB2 encoding", section => 'G'     },
+            { name => '_menu_memory',   text => "- Menu memory",              section => 'G'     },
+            { name => '_table_expand',  text => "- Expand table",             section => 'table' },
+            { name => '_search',        text => "- Search",                   section => 'table' },
+            { name => '_mouse',         text => "- Mouse mode",               section => 'table' },
+            { name => '_db2_encoding',  text => "- DB2 encoding",             section => 'G'     },
         ],
     };
     return $groups->{$group_name};
@@ -338,11 +338,11 @@ sub set_options {
             elsif ( $opt eq '_alias' ) {
                 my $prompt = 'Enable alias for:';
                 my $sub_menu = [
-                    [ 'select_complex_col', "- Functions/Subqueries in SELECT",  [ $no, 'ASK'    ] ],
-                    [ 'join_table',         "- Tables in join",                  [ 'AUTO', 'ASK' ] ],
-                    [ 'join_columns',       "- Non-unique columns in join",      [ $no, 'AUTO'   ] ],
-                    [ 'derived_table',      "- Derived table",                   [ 'AUTO', 'ASK' ] ],
-                    [ 'table',              "- Ordinary table",                  [ $no, 'AUTO'   ] ],
+                    [ 'select_complex_col', "- Functions/Subqueries in SELECT",  [ 'NO',   undef, 'ASK',   undef     ] ],
+                    [ 'join_table',         "- Tables in join",                  [ undef, 'AUTO',  undef, 'ASK/AUTO' ] ],
+                    [ 'join_columns',       "- Non-unique columns in join",      [ 'NO',  'AUTO', 'ASK',  'ASK/AUTO' ] ],
+                    [ 'derived_table',      "- Derived table",                   [ 'NO',  'AUTO', 'ASK',  'ASK/AUTO' ] ],
+                    [ 'table',              "- Ordinary table",                  [ 'NO',  'AUTO', 'ASK',  'ASK/AUTO' ] ],
                 ];
                 $sf->__settings_menu_wrap( $section, $sub_menu, $prompt );
             }
