@@ -5,11 +5,12 @@ use warnings;
 use strict;
 use 5.014;
 
-use Term::Choose           qw();
-use Term::Form::ReadLine   qw();
+use Term::Choose         qw();
+use Term::Form::ReadLine qw();
 
 use App::DBBrowser::Auxil;
 use App::DBBrowser::Table::Extensions;
+use App::DBBrowser::Table::Substatements;
 
 
 sub new {
@@ -35,7 +36,7 @@ sub maths {
     my $prompt = $opt->{prompt} // 'Math:';
     my @bu;
 
-    COLUMNS: while ( 1 ) { ##
+    CHOICE: while ( 1 ) {
         my $fill_string = join( ' ', @$items, '?' );
         $fill_string =~ s/\(\s/(/g;
         $fill_string =~ s/\s\)/)/g;
@@ -48,7 +49,7 @@ sub maths {
         if ( ! $idx ) {
             if ( @bu ) {
                 $items = pop @bu;
-                next COLUMNS;
+                next CHOICE;
             }
             return;
         }
@@ -68,7 +69,7 @@ sub maths {
                 { from => 'maths', info => $tmp_info }
             );
             if ( ! defined $complex_col ) {
-                next COLUMNS;
+                next CHOICE;
             }
             push @bu, [ @$items ];
             push @$items, $complex_col;
@@ -80,7 +81,7 @@ sub maths {
                 { %{$sf->{i}{lyt_h}}, info => $tmp_info . "\n" . $prompt, prompt => '', undef => '<=' }
             );
             if ( ! defined $operator ) {
-                next COLUMNS;
+                next CHOICE;
             }
             push @bu, [ @$items ];
             push @$items, $operator =~ s/^\s+|\s+\z//gr;
@@ -91,14 +92,20 @@ sub maths {
                 { info => $tmp_info . "\n" . $prompt }
             );
             if ( ! length $number ) {
-                next COLUMNS;
+                next CHOICE;
             }
             push @bu, [ @$items ];
             push @$items, $number;
         }
         else {
             push @bu, [ @$items ];
-            push @$items, $menu->[$idx];
+            if ( $sql->{aggregate_mode} && $clause =~ /^(?:having|order_by)\z/ ) {
+                my $sb = App::DBBrowser::Table::Substatements->new( $sf->{i}, $sf->{o}, $sf->{d} );
+                push @$items, $sb->get_prepared_aggr_func( $sql, $clause, $menu->[$idx] );
+            }
+            else {
+                push @$items, $menu->[$idx];
+            }
         }
     }
 }
